/*
 * RealThumb v0.21
 * CoRTContext.cpp - RealPlayer Context implementation
 * Copyright (c) 2003 Michael Barrientos
 * All rights reserved.
 * http://mb.n3.net/projects/
 */

// CoRTContext.cpp : Implementation of CCoRTContext

#include "stdafx.h"
#include "CoRTContext.h"

// CCoRTContext
/*	ICoRTContext methods */
STDMETHODIMP
CCoRTContext::Init(THIS_
	IUnknown* pPlayer,
	ICoRealThumb* pRealThumb)
{
	m_RealThumb = pRealThumb;
	m_Player = pPlayer;
	if (!m_Player)
	{
		return E_NOINTERFACE;
	}
	m_SiteManager = m_Player;
	if (!m_SiteManager)
	{
		return E_NOINTERFACE;
	}
	m_Player->AddAdviseSink(this);

	return S_OK;
}

/*
 *	IRMAClientAdviseSink methods
 */

/************************************************************************
 *	Method:
 *		IRMAClientAdviseSink::OnPosLength
 *	Purpose:
 *		Called to advise the client that the position or length of the
 *		current playback context has changed.
 */
STDMETHODIMP
CCoRTContext::OnPosLength(THIS_
				UINT32	  ulPosition,
				UINT32	  ulLength)
{
	ATLTRACE2("ClientAdviseSink: OnPosLength\n");
	return PNR_OK;
}

/************************************************************************
 *	Method:
 *		IRMAClientAdviseSink::OnPresentationOpened
 *	Purpose:
 *		Called to advise the client a presentation has been opened.
 */
STDMETHODIMP
CCoRTContext::OnPresentationOpened(THIS)
{
	ATLTRACE2("ClientAdviseSink: OnPresentationOpened\n");
	return PNR_OK;
}

/************************************************************************
 *	Method:
 *		IRMAClientAdviseSink::OnPresentationClosed
 *	Purpose:
 *		Called to advise the client a presentation has been closed.
 */
STDMETHODIMP
CCoRTContext::OnPresentationClosed(THIS)
{
	ATLTRACE2("ClientAdviseSink: OnPresentationClosed\n");
	return PNR_OK;
}

/************************************************************************
 *	Method:
 *		IRMAClientAdviseSink::OnStatisticsChanged
 *	Purpose:
 *		Called to advise the client that the presentation statistics
 *		have changed. 
 */
STDMETHODIMP
CCoRTContext::OnStatisticsChanged(THIS)
{
	ATLTRACE2("ClientAdviseSink: OnStatisticsChanged\n");
	return PNR_OK;
}

/************************************************************************
 *	Method:
 *		IRMAClientAdviseSink::OnPreSeek
 *	Purpose:
 *		Called by client engine to inform the client that a seek is
 *		about to occur. The render is informed the last time for the 
 *		stream's time line before the seek, as well as the first new
 *		time for the stream's time line after the seek will be completed.
 *
 */
STDMETHODIMP
CCoRTContext::OnPreSeek(THIS_
			ULONG32			ulOldTime,
			ULONG32			ulNewTime)
{
	ATLTRACE2("ClientAdviseSink: OnPreSeek\n");
	return PNR_OK;
}

/************************************************************************
 *	Method:
 *		IRMAClientAdviseSink::OnPostSeek
 *	Purpose:
 *		Called by client engine to inform the client that a seek has
 *		just occurred. The render is informed the last time for the 
 *		stream's time line before the seek, as well as the first new
 *		time for the stream's time line after the seek.
 *
 */
STDMETHODIMP
CCoRTContext::OnPostSeek(THIS_
			ULONG32			ulOldTime,
			ULONG32			ulNewTime)
{
	ATLTRACE2("ClientAdviseSink: OnPostSeek\n");
	return PNR_OK;
}

/************************************************************************
 *	Method:
 *		IRMAClientAdviseSink::OnStop
 *	Purpose:
 *		Called by client engine to inform the client that a stop has
 *		just occurred. 
 *
 */
STDMETHODIMP
CCoRTContext::OnStop(THIS)
{
	ATLTRACE2("ClientAdviseSink: OnStop\n");
	return PNR_OK;
}

/************************************************************************
 *	Method:
 *		IRMAClientAdviseSink::OnPause
 *	Purpose:
 *		Called by client engine to inform the client that a pause has
 *		just occurred. The render is informed the last time for the 
 *		stream's time line before the pause.
 *
 */
STDMETHODIMP
CCoRTContext::OnPause(THIS_
			ULONG32			ulTime)
{
	ATLTRACE2("ClientAdviseSink: OnPause\n");
	return PNR_OK;
}

/************************************************************************
 *	Method:
 *		IRMAClientAdviseSink::OnBegin
 *	Purpose:
 *		Called by client engine to inform the client that a begin or
 *		resume has just occurred. The render is informed the first time 
 *		for the stream's time line after the resume.
 *
 */
STDMETHODIMP
CCoRTContext::OnBegin(THIS_
			ULONG32			ulTime)
{
	ATLTRACE2("ClientAdviseSink: OnBegin\n");
	UINT16 numSources = m_Player->GetSourceCount();
	BOOL foundVideoStream = FALSE;
	ATLTRACE2("Number of Sources: %d\n", numSources);
	for (UINT i = 0; !foundVideoStream && i < numSources; i++)
	{
		CComPtr<IUnknown> tmp_SourceUnknown;
		if (m_Player->GetSource(i, tmp_SourceUnknown.p) == PNR_OK)
		{
			CComQIPtr<IRMAStreamSource, &IID_IRMAStreamSource> tmp_Source = tmp_SourceUnknown;
			if (tmp_Source)
			{
				UINT16 numStreams = tmp_Source->GetStreamCount();
				ATLTRACE2("Number of Streams in Source %d: %d\n", i, numStreams);
				for (UINT16 j = 0; !foundVideoStream && j < numStreams; j++)
				{
					CComPtr<IUnknown> tmp_StreamUnknown;
					if (tmp_Source->GetStream(j, tmp_StreamUnknown.p) == PNR_OK)
					{
						CComQIPtr<IRMAStream, &IID_IRMAStream> tmp_Stream = tmp_StreamUnknown;
						if (tmp_Stream)
						{
							LPCSTR tmp_StreamType = tmp_Stream->GetStreamType();
							ATLTRACE2("Source %d, Stream %d: %s\n", i, j, tmp_StreamType);
							if (StrCSpnI(tmp_StreamType, "video/") == 0)
							{
								foundVideoStream = TRUE;
							}
						}
					}
				}
			}
		}
	}
	if (!foundVideoStream)
	{
		ATLTRACE2("ClientAdviseSink: OnBegin - No video stream\n");
		// Stop it, audio only
		HRESULT hr = m_RealThumb->ImageCaptured(NULL, NULL);
	}

    return PNR_OK;
}

/************************************************************************
 *	Method:
 *		IRMAClientAdviseSink::OnBuffering
 *	Purpose:
 *		Called by client engine to inform the client that buffering
 *		of data is occuring. The render is informed of the reason for
 *		the buffering (start-up of stream, seek has occurred, network
 *		congestion, etc.), as well as percentage complete of the 
 *		buffering process.
 *
 */
STDMETHODIMP
CCoRTContext::OnBuffering(THIS_
			ULONG32			ulFlags,
			UINT16			unPercentComplete)
{
	ATLTRACE2("ClientAdviseSink: OnBuffering\n");
	return PNR_OK;
}

/************************************************************************
 *	Method:
 *		IRMAClientAdviseSink::OnContacting
 *	Purpose:
 *		Called by client engine to inform the client is contacting
 *		hosts(s).
 *
 */
STDMETHODIMP
CCoRTContext::OnContacting(THIS_
			 const char*	pHostName)
{
	ATLTRACE2("ClientAdviseSink: OnContacting\n");
	return PNR_OK;
}

/*
 * IRMASiteSupplier methods
 */

/************************************************************************
 *	Method:
 *		IRMASiteSupplier::SitesNeeded
 *	Purpose:
 *		Called to inform the site supplier that a site with a particular
 *		set of characteristics is needed. If the site supplier can 
 *		fulfill the request it should call the site manager and add one
 *		or more new sites.
 *		Note that the request for sites is associated with a Request ID
 *		the client core will inform the site supplier when this requested
 *		site is no longer needed.
 */
STDMETHODIMP
CCoRTContext::SitesNeeded(THIS_
			UINT32			uRequestID,
			IRMAValues*		pSiteProps)
{
	ATLTRACE2("SiteSupplier: SitesNeeded\n");
	if (!pSiteProps)
	{
		return PNR_INVALID_PARAMETER;
	}

	CComQIPtr<ICoRTSite, &IID_ICoRTSite> tmp_RTSite = new CCOMOBJECT<CCoRTSite>;
	if (!tmp_RTSite)
	{
		return PNR_OUTOFMEMORY;
	}

	CComQIPtr<IRMASite, &IID_IRMASite> tmp_Site = tmp_RTSite;
	if (!tmp_Site)
	{
		return PNR_NOINTERFACE;
	}

	tmp_RTSite->Init(m_Player, m_RealThumb);

	CComQIPtr<IRMAValues, &IID_IRMAValues> tmp_SiteProps = tmp_Site;
	if (!pSiteProps)
	{
		return PNR_NOINTERFACE;
	}

	// Set values (from Example Site Supplier)
	CComQIPtr<IRMABuffer, &IID_IRMABuffer> tmp_Buffer;
	HRESULT hres = pSiteProps->GetPropertyCString("playto", tmp_Buffer.p);
	if (PNR_OK == hres)
	{
		tmp_SiteProps->SetPropertyCString("channel",tmp_Buffer);
	}
	else
	{
		hres = pSiteProps->GetPropertyCString("name",tmp_Buffer.p);
		if (PNR_OK != hres)
		{
			return hres;
		}
		tmp_SiteProps->SetPropertyCString("LayoutGroup",tmp_Buffer);
	}

	hres = m_SiteManager->AddSite(tmp_Site);
	if (PNR_OK != hres)
	{
		return hres;
	}

	m_Sites.SetAt(uRequestID, tmp_Site);

	return PNR_OK;
}

/************************************************************************
 *  Method:
 *	IRMASiteSupplier::SitesNotNeeded
 *  Purpose:
 *	Called to inform the site supplier that all sites from a previos
 *	  site request are no longer needed. If the site supplier had 
 *	  previously created non-persistant sites (like popup windows)
 *	to fulfill a request for sites it should call the site manager 
 *	and remove those sites.
 */
STDMETHODIMP
CCoRTContext::SitesNotNeeded(THIS_
			UINT32			uRequestID)
{
	ATLTRACE2("SiteSupplier: SitesNotNeeded\n");
	CComQIPtr<IRMASite, &IID_IRMASite> tmp_Site;
	if (!m_Sites.Lookup(uRequestID, tmp_Site))
	{
		return PNR_INVALID_PARAMETER;
	}

	m_SiteManager->RemoveSite(tmp_Site);
	m_Sites.RemoveKey(uRequestID);
	return PNR_OK;
}


/************************************************************************
 *  Method:
 *	IRMASiteSupplier::BeginChangeLayout
 *  Purpose:
 *	Called to inform the site supplier a layout change has beginning
 *	  it can expect to recieve SitesNeeded() and SitesNotNeeded() calls
 *	  while a layout change is in progress,
 */
STDMETHODIMP
CCoRTContext::BeginChangeLayout(THIS)
{
	ATLTRACE2("SiteSupplier: BeginChangeLayout\n");
	return PNR_OK;
}

/************************************************************************
 *  Method:
 *	IRMASiteSupplier::DoneChangeLayout
 *  Purpose:
 *	Called to inform the site supplier the layout change has been
 *	  completed.
 */
STDMETHODIMP
CCoRTContext::DoneChangeLayout(THIS)
{
	ATLTRACE2("SiteSupplier: DoneChangeLayout\n");
	return PNR_OK;
}