/*
 * RealThumb v0.21
 * CoRTContext.h - RealPlayer Context header
 * Copyright (c) 2003 Michael Barrientos
 * All rights reserved.
 * http://mb.n3.net/projects/
 */

// CoRTContext.h : Declaration of the CCoRTContext

#pragma once
#include "resource.h"	   // main symbols

#include "RealThumb.h"
#include "CoRTSite.h"

// CCoRTContext

class ATL_NO_VTABLE CCoRTContext : 
	public CComObjectRootEx<CComObjectThreadModel>,
	public CComCoClass<CCoRTContext, &CLSID_CoRTContext>,
	public ICoRTContext,
	public IRMAClientAdviseSink,
	public IRMASiteSupplier
{
protected:
	CComPtr<ICoRealThumb> m_RealThumb;
	CComQIPtr<IRMAPlayer, &IID_IRMAPlayer> m_Player;
	CComQIPtr<IRMASiteManager, &IID_IRMASiteManager> m_SiteManager;
	CAtlMap<UINT32,	CComQIPtr<IRMASite, &IID_IRMASite>,
		CElementTraits<UINT32>,	CComQIPtrElementTraits<IRMASite, &IID_IRMASite> > m_Sites;

public:
	CCoRTContext()
	{
	}

DECLARE_REGISTRY_RESOURCEID(IDR_CORTCONTEXT)

BEGIN_COM_MAP(CCoRTContext)
	COM_INTERFACE_ENTRY(ICoRTContext)
	COM_INTERFACE_ENTRY_IID(IID_IRMAClientAdviseSink, IRMAClientAdviseSink)
	COM_INTERFACE_ENTRY_IID(IID_IRMASiteSupplier, IRMASiteSupplier)
END_COM_MAP()

	HRESULT FinalConstruct()
	{
		return S_OK;
	}

	void FinalRelease()
	{
		m_RealThumb = NULL;
		m_Player = reinterpret_cast<IUnknown*>(NULL);
		m_SiteManager = reinterpret_cast<IUnknown*>(NULL);
		m_Sites.RemoveAll();
	}

public:
	/*	ICoRTContext methods */
	STDMETHOD(Init)	(THIS_
		IUnknown* pPlayer,
		ICoRealThumb* pRealThumb);
	
	/*
	 *	IRMAClientAdviseSink methods
	 */

	/************************************************************************
	 *	Method:
	 *		IRMAClientAdviseSink::OnPosLength
	 *	Purpose:
	 *		Called to advise the client that the position or length of the
	 *		current playback context has changed.
	 */
	STDMETHOD(OnPosLength)		(THIS_
					UINT32	  ulPosition,
					UINT32	  ulLength);

	/************************************************************************
	 *	Method:
	 *		IRMAClientAdviseSink::OnPresentationOpened
	 *	Purpose:
	 *		Called to advise the client a presentation has been opened.
	 */
	STDMETHOD(OnPresentationOpened)	(THIS);

	/************************************************************************
	 *	Method:
	 *		IRMAClientAdviseSink::OnPresentationClosed
	 *	Purpose:
	 *		Called to advise the client a presentation has been closed.
	 */
	STDMETHOD(OnPresentationClosed)	(THIS);

	/************************************************************************
	 *	Method:
	 *		IRMAClientAdviseSink::OnStatisticsChanged
	 *	Purpose:
	 *		Called to advise the client that the presentation statistics
	 *		have changed. 
	 */
	STDMETHOD(OnStatisticsChanged)	(THIS);

	/************************************************************************
	 *	Method:
	 *		IRMAClientAdviseSink::OnPreSeek
	 *	Purpose:
	 *		Called by client engine to inform the client that a seek is
	 *		about to occur. The render is informed the last time for the 
	 *		stream's time line before the seek, as well as the first new
	 *		time for the stream's time line after the seek will be completed.
	 *
	 */
	STDMETHOD(OnPreSeek)	(THIS_
				ULONG32			ulOldTime,
				ULONG32			ulNewTime);

	/************************************************************************
	 *	Method:
	 *		IRMAClientAdviseSink::OnPostSeek
	 *	Purpose:
	 *		Called by client engine to inform the client that a seek has
	 *		just occurred. The render is informed the last time for the 
	 *		stream's time line before the seek, as well as the first new
	 *		time for the stream's time line after the seek.
	 *
	 */
	STDMETHOD(OnPostSeek)	(THIS_
				ULONG32			ulOldTime,
				ULONG32			ulNewTime);

	/************************************************************************
	 *	Method:
	 *		IRMAClientAdviseSink::OnStop
	 *	Purpose:
	 *		Called by client engine to inform the client that a stop has
	 *		just occurred. 
	 *
	 */
	STDMETHOD(OnStop)		(THIS);

	/************************************************************************
	 *	Method:
	 *		IRMAClientAdviseSink::OnPause
	 *	Purpose:
	 *		Called by client engine to inform the client that a pause has
	 *		just occurred. The render is informed the last time for the 
	 *		stream's time line before the pause.
	 *
	 */
	STDMETHOD(OnPause)		(THIS_
				ULONG32			ulTime);

	/************************************************************************
	 *	Method:
	 *		IRMAClientAdviseSink::OnBegin
	 *	Purpose:
	 *		Called by client engine to inform the client that a begin or
	 *		resume has just occurred. The render is informed the first time 
	 *		for the stream's time line after the resume.
	 *
	 */
	STDMETHOD(OnBegin)		(THIS_
				ULONG32			ulTime);

	/************************************************************************
	 *	Method:
	 *		IRMAClientAdviseSink::OnBuffering
	 *	Purpose:
	 *		Called by client engine to inform the client that buffering
	 *		of data is occuring. The render is informed of the reason for
	 *		the buffering (start-up of stream, seek has occurred, network
	 *		congestion, etc.), as well as percentage complete of the 
	 *		buffering process.
	 *
	 */
	STDMETHOD(OnBuffering)	(THIS_
				ULONG32			ulFlags,
				UINT16			unPercentComplete);

	/************************************************************************
	 *	Method:
	 *		IRMAClientAdviseSink::OnContacting
	 *	Purpose:
	 *		Called by client engine to inform the client is contacting
	 *		hosts(s).
	 *
	 */
	STDMETHOD(OnContacting)	(THIS_
				 const char*	pHostName);

	/*
	 * IRMASiteSupplier methods
	 */

	/************************************************************************
	 *	Method:
	 *		IRMASiteSupplier::SitesNeeded
	 *	Purpose:
	 *		Called to inform the site supplier that a site with a particular
	 *		set of characteristics is needed. If the site supplier can 
	 *		fulfill the request it should call the site manager and add one
	 *		or more new sites.
	 *		Note that the request for sites is associated with a Request ID
	 *		the client core will inform the site supplier when this requested
	 *		site is no longer needed.
	 */
	STDMETHOD(SitesNeeded)	(THIS_
				UINT32			uReqestID,
				IRMAValues*		pSiteProps);

	/************************************************************************
	 *  Method:
	 *	IRMASiteSupplier::SitesNotNeeded
	 *  Purpose:
	 *	Called to inform the site supplier that all sites from a previos
	 *	  site request are no longer needed. If the site supplier had 
	 *	  previously created non-persistant sites (like popup windows)
	 *	to fulfill a request for sites it should call the site manager 
	 *	and remove those sites.
	 */
	STDMETHOD(SitesNotNeeded)	(THIS_
				UINT32			uReqestID);


	/************************************************************************
	 *  Method:
	 *	IRMASiteSupplier::BeginChangeLayout
	 *  Purpose:
	 *	Called to inform the site supplier a layout change has beginning
	 *	  it can expect to recieve SitesNeeded() and SitesNotNeeded() calls
	 *	  while a layout change is in progress,
	 */
	STDMETHOD(BeginChangeLayout) (THIS);

	/************************************************************************
	 *  Method:
	 *	IRMASiteSupplier::DoneChangeLayout
	 *  Purpose:
	 *	Called to inform the site supplier the layout change has been
	 *	  completed.
	 */
	STDMETHOD(DoneChangeLayout) (THIS);
};

OBJECT_ENTRY_AUTO(__uuidof(CoRTContext), CCoRTContext)
