/*
 * RealThumb v0.21
 * CoRTNullAudioDevice.cpp - RealPlayer dump-into-a-black-hole audio device implementation
 * Copyright (c) 2003 Michael Barrientos
 * All rights reserved.
 * http://mb.n3.net/projects/
 */

// CoRTNullAudioDevice.cpp : Implementation of CCoRTNullAudioDevice

#include "stdafx.h"
#include "CoRTNullAudioDevice.h"

// CCoRTNullAudioDevice

/*
 *  IRMAAudioDevice methods
 */

/************************************************************************
*  Method:
*		IRMAAudioDevice::Open
*  Purpose:
*		The caller calls this to open the audio device using the audio
*		format given.
*/
STDMETHODIMP
CCoRTNullAudioDevice::Open(THIS_
	const RMAAudioFormat*	/*IN*/ pAudioFormat,
	IRMAAudioDeviceResponse*	/*IN*/ pStreamResponse)
{
	ATLTRACE2("AudioDevice: Open\n");
	return PNR_OK;
};

/************************************************************************
*  Method:
*	  IRMAAudioDevice::Close
*  Purpose:
*		The caller calls this to close the audio device.
*/
STDMETHODIMP
CCoRTNullAudioDevice::Close(THIS_
		const BOOL  /*IN*/ bFlush )
{
	ATLTRACE2("AudioDevice: Open\n");
	return PNR_OK;
}

/************************************************************************
*  Method:
*		IRMAAudioDevice::Resume
*  Purpose:
*		The caller calls this to start or resume audio playback.
*/
STDMETHODIMP
CCoRTNullAudioDevice::Resume(THIS)
{
	ATLTRACE2("AudioDevice: Resume\n");
	return PNR_OK;
}

/************************************************************************
*  Method:
*		IRMAAudioDevice::Pause
*  Purpose:
*		The caller calls this to pause the audio device. If bFlush is
*		TRUE, any buffers in the audio device will be flushed; otherwise,
*		the buffers are played.
*/
STDMETHODIMP
CCoRTNullAudioDevice::Pause(THIS)
{
	ATLTRACE2("AudioDevice: Pause\n");
	return PNR_OK;
}

/************************************************************************
*  Method:
*		IRMAAudioDevice::Write
*  Purpose:
*		The caller calls this to write an audio buffer.
*/
STDMETHODIMP
CCoRTNullAudioDevice::Write (THIS_
			 const RMAAudioData* /*IN*/ pAudioData)
{
	ATLTRACE2("AudioDevice: Write\n");
	return PNR_OK;
}

/************************************************************************
*  Method:
*	  IRMAAudioDevice::InitVolume
*  Purpose:
*		The caller calls this to inform the audio stream of the client's
*		volume range. The audio stream maps the client's volume range
*		into the audio device volume range. 
*		NOTE: This function returns TRUE if volume is supported by this 
*		audio device.
*/
STDMETHODIMP_(BOOL)
CCoRTNullAudioDevice::InitVolume(THIS_
			 const UINT16	/*IN*/ uMinVolume,
			 const UINT16	/*IN*/ uMaxVolume)
{
	ATLTRACE2("AudioDevice: InitVolume\n");
	return FALSE;
}

/************************************************************************
*  Method:
*		IRMAAudioDevice::SetVolume
*  Purpose:
*		The caller calls this to set the audio device volume level.
*/
STDMETHODIMP
CCoRTNullAudioDevice::SetVolume(THIS_
			 const UINT16	/*IN*/ uVolume)
{
	ATLTRACE2("AudioDevice: SetVolume\n");
	return E_FAIL;
}

/************************************************************************
*  Method:
*		IRMAAudioDevice::GetVolume
*  Purpose:
*		The caller calls this to get the audio device volume level.
*/
STDMETHODIMP_(UINT16)
CCoRTNullAudioDevice::GetVolume(THIS)
{
	ATLTRACE2("AudioDevice: GetVolume\n");
	return 0;
}

/************************************************************************
*  Method:
*		IRMAAudioDevice::Reset
*  Purpose:
*		The caller calls this to reset the audio device.
*/
STDMETHODIMP
CCoRTNullAudioDevice::Reset(THIS)
{
	ATLTRACE2("AudioDevice: Reset\n");
	return PNR_OK;
}

/************************************************************************
*  Method:
*		IRMAAudioDevice::Drain
*  Purpose:
*		The caller calls this to drain the audio device.
*/
STDMETHODIMP
CCoRTNullAudioDevice::Drain(THIS)
{
	// No audio, so nothing to drain
	ATLTRACE2("AudioDevice: Drain\n");
	return PNR_OK;
}

/************************************************************************
*  Method:
*		IRMAAudioDevice::CheckFormat
*  Purpose:
*		The caller calls this to check the input format with the
*		audio device format.
*/
STDMETHODIMP
CCoRTNullAudioDevice::CheckFormat(THIS_
			const RMAAudioFormat* /*IN*/ pAudioFormat )
{
	// We don't care what format it's in since we throw it all away anyways.
	ATLTRACE2("AudioDevice: CheckFormat\n");
	return PNR_OK;
}

/************************************************************************
*  Method:
*		IRMAAudioDevice::GetCurrentAudioTime
*  Purpose:
*		The caller calls this to get current system audio time.
*/
STDMETHODIMP
CCoRTNullAudioDevice::GetCurrentAudioTime(THIS_
				REF(ULONG32) /*OUT*/ ulCurrentTime)
{
	ATLTRACE2("AudioDevice: GetCurrentAudioTime\n");
	ulCurrentTime = 0;
	return PNR_OK;
}