/*
 * RealThumb v0.21
 * CoRTNullAudioDevice.h - RealPlayer dump-into-a-black-hole audio device header
 * Copyright (c) 2003 Michael Barrientos
 * All rights reserved.
 * http://mb.n3.net/projects/
 */

// CoRTNullAudioDevice.h : Declaration of the CCoRTNullAudioDevice

#pragma once
#include "resource.h"	   // main symbols

#include "RealThumb.h"
#include "rmaausvc.h"

// CCoRTNullAudioDevice

class ATL_NO_VTABLE CCoRTNullAudioDevice : 
	public CComObjectRootEx<CComObjectThreadModel>,
	public CComCoClass<CCoRTNullAudioDevice, &CLSID_CoRTNullAudioDevice>,
	public ICoRTNullAudioDevice,
	public IRMAAudioDevice
{
public:
	CCoRTNullAudioDevice()
	{
	}

	~CCoRTNullAudioDevice()
	{
	}

DECLARE_REGISTRY_RESOURCEID(IDR_CORTNULLAUDIODEVICE)


BEGIN_COM_MAP(CCoRTNullAudioDevice)
	COM_INTERFACE_ENTRY(ICoRTNullAudioDevice)
	COM_INTERFACE_ENTRY_IID(IID_IRMAAudioDevice, IRMAAudioDevice)
END_COM_MAP()


	DECLARE_PROTECT_FINAL_CONSTRUCT()
	DECLARE_GET_CONTROLLING_UNKNOWN()

	HRESULT FinalConstruct()
	{
		return S_OK;
	}

	void FinalRelease()
	{
	}

public:
	/*
	 *  IRMAAudioDevice methods
	 */

	/************************************************************************
	*  Method:
	*		IRMAAudioDevice::Open
	*  Purpose:
	*		The caller calls this to open the audio device using the audio
	*		format given.
	*/
	STDMETHOD(Open) (THIS_
			const RMAAudioFormat*	/*IN*/ pAudioFormat,
			IRMAAudioDeviceResponse*	/*IN*/ pStreamResponse);

	/************************************************************************
	*  Method:
	*	  IRMAAudioDevice::Close
	*  Purpose:
	*		The caller calls this to close the audio device.
	*/
	STDMETHOD(Close)	(THIS_
			const BOOL  /*IN*/ bFlush );

	/************************************************************************
	*  Method:
	*		IRMAAudioDevice::Resume
	*  Purpose:
	*		The caller calls this to start or resume audio playback.
	*/
	STDMETHOD(Resume)		 (THIS);

	/************************************************************************
	*  Method:
	*		IRMAAudioDevice::Pause
	*  Purpose:
	*		The caller calls this to pause the audio device. If bFlush is
	*		TRUE, any buffers in the audio device will be flushed; otherwise,
	*		the buffers are played.
	*/
	STDMETHOD(Pause)		 (THIS);

	/************************************************************************
	*  Method:
	*		IRMAAudioDevice::Write
	*  Purpose:
	*		The caller calls this to write an audio buffer.
	*/
	STDMETHOD(Write)		 (THIS_
				 const RMAAudioData* /*IN*/ pAudioData);

	/************************************************************************
	*  Method:
	*	  IRMAAudioDevice::InitVolume
	*  Purpose:
	*		The caller calls this to inform the audio stream of the client's
	*		volume range. The audio stream maps the client's volume range
	*		into the audio device volume range. 
	*		NOTE: This function returns TRUE if volume is supported by this 
	*		audio device.
	*/
	STDMETHOD_(BOOL,InitVolume)  (THIS_
				 const UINT16	/*IN*/ uMinVolume,
				 const UINT16	/*IN*/ uMaxVolume);

	/************************************************************************
	*  Method:
	*		IRMAAudioDevice::SetVolume
	*  Purpose:
	*		The caller calls this to set the audio device volume level.
	*/
	STDMETHOD(SetVolume)		 (THIS_
				 const UINT16	/*IN*/ uVolume);

	/************************************************************************
	*  Method:
	*		IRMAAudioDevice::GetVolume
	*  Purpose:
	*		The caller calls this to get the audio device volume level.
	*/
	STDMETHOD_(UINT16,GetVolume) (THIS);

	/************************************************************************
	*  Method:
	*		IRMAAudioDevice::Reset
	*  Purpose:
	*		The caller calls this to reset the audio device.
	*/
	STDMETHOD(Reset)		(THIS);

	/************************************************************************
	*  Method:
	*		IRMAAudioDevice::Drain
	*  Purpose:
	*		The caller calls this to drain the audio device.
	*/
	STDMETHOD(Drain)		(THIS);

	/************************************************************************
	*  Method:
	*		IRMAAudioDevice::CheckFormat
	*  Purpose:
	*		The caller calls this to check the input format with the
	*		audio device format.
	*/
	STDMETHOD(CheckFormat)  (THIS_
				const RMAAudioFormat* /*IN*/ pAudioFormat );

	/************************************************************************
	*  Method:
	*		IRMAAudioDevice::GetCurrentAudioTime
	*  Purpose:
	*		The caller calls this to get current system audio time.
	*/
	STDMETHOD(GetCurrentAudioTime)  (THIS_
					REF(ULONG32) /*OUT*/ ulCurrentTime);
};

OBJECT_ENTRY_AUTO(__uuidof(CoRTNullAudioDevice), CCoRTNullAudioDevice)