/*
 * RealThumb v0.21
 * CoRTSite.cpp - RealPlayer Site implementation
 * Copyright (c) 2003 Michael Barrientos
 * All rights reserved.
 * http://mb.n3.net/projects/
 */

// CoRTSite.cpp : Implementation of CCoRTSite

#include "stdafx.h"
#include "CoRTSite.h"

// CCoRTSite
/* ICoRTSite methods */
STDMETHODIMP
CCoRTSite::Init(THIS_
			IUnknown *pContext,
			ICoRealThumb *pRealThumb)
{
	ATLTRACE2("CoRTSite: Init\n");
	m_RealThumb = pRealThumb;
	CComQIPtr<IRMACommonClassFactory, &IID_IRMACommonClassFactory> tmp_CCF = pContext;
	tmp_CCF->CreateInstance(CLSID_IRMAValues, (void**)&m_Values.p);
	return S_OK;
}

/*
 * IRMASite methods usually called by the "context" to 
 * associate users with the site, and to create child sites
 * as appropriate.
 */
STDMETHODIMP
CCoRTSite::AttachUser(THIS_
			IRMASiteUser*	/*IN*/	pUser)
{
	ATLTRACE2("Site: AttachUser\n");
	if (m_SiteUser)
	{
		return PNR_UNEXPECTED;
	}

	HRESULT hr = pUser->AttachSite(this);
	if (hr != PNR_OK)
	{
		return hr;
	}
	m_SiteUser = pUser;
	return PNR_OK;
}

STDMETHODIMP
CCoRTSite::DetachUser(THIS)
{
	ATLTRACE2("Site: DetachUser\n");
	if (!m_SiteUser)
	{
		return PNR_UNEXPECTED;
	}

	HRESULT hr = m_SiteUser->DetachSite();
	if (hr != PNR_OK)
	{
		return hr;
	}
	m_SiteUser = static_cast<IUnknown*>(NULL);

	return PNR_OK;
}


STDMETHODIMP
CCoRTSite::GetUser(THIS_
			REF(IRMASiteUser*) /*OUT*/ pUser)
{
	ATLTRACE2("Site: GetUser\n");
	if (!m_SiteUser)
	{
		return PNR_UNEXPECTED;
	}

	m_SiteUser->QueryInterface(IID_IRMASiteUser, reinterpret_cast<void**>(&pUser));

	return PNR_OK;
}

STDMETHODIMP
CCoRTSite::CreateChild(THIS_
			REF(IRMASite*)	/*OUT*/ pChildSite)
{
	ATLTRACE2("Site: CreateChild\n");
	return PNR_NOTIMPL;
}

STDMETHODIMP
CCoRTSite::DestroyChild(THIS_
			IRMASite*	/*IN*/	pChildSite)
{
	ATLTRACE2("Site: DestroyChild\n");
	return PNR_NOTIMPL;
}

/*
 * IRMASite methods called by the the "context" in which the site
 * is displayed in order to manage its position. Site users should
 * not generally call these methods.
 */
STDMETHODIMP
CCoRTSite::AttachWatcher(THIS_
			IRMASiteWatcher* /*IN*/	pWatcher)
{
	ATLTRACE2("Site: AttachWatcher\n");
/*
	if (m_SiteWatcher)
	{
		return PNR_UNEXPECTED;
	}

	HRESULT hr = pWatcher->AttachSite(this);
	if (hr != PNR_OK)
	{
		return hr;
	}
	m_SiteWatcher = pWatcher;
*/
	return PNR_OK;
}

STDMETHODIMP
CCoRTSite::DetachWatcher(THIS)
{
	ATLTRACE2("Site: DetachWatcher\n");
/*
	if (!m_SiteWatcher)
	{
		return PNR_UNEXPECTED;
	}

	HRESULT hr = m_SiteWatcher->DetachSite();
	if (hr != PNR_OK)
	{
		return hr;
	}
	m_SiteWatcher = static_cast<IUnknown*>(NULL);
*/
	return PNR_OK;
}

STDMETHODIMP
CCoRTSite::SetPosition(THIS_
			PNxPoint		position)
{
	ATLTRACE2("Site: SetPosition\n");
/*
	HRESULT hr = PNR_OK;
	if (m_SiteWatcher)
	{
		hr = m_SiteWatcher->ChangingPosition(m_Position, position);
	}
	if (hr == PNR_OK)
	{
		m_Position = position;
		// Iterate through passive site watchers
		POSITION pos = m_PassiveSiteWatchers.GetHeadPosition();
		while (pos != NULL)
		{
			CComQIPtr<IRMAPassiveSiteWatcher, &IID_IRMAPassiveSiteWatcher> tmp_Watcher =
				m_PassiveSiteWatchers.GetNext(pos);
			tmp_Watcher->PositionChanged(&m_Position);
		}
	}

	return hr;
*/
	return PNR_OK;
}

STDMETHODIMP
CCoRTSite::GetPosition(THIS_
			REF(PNxPoint)		position)
{
	ATLTRACE2("Site: GetPostion\n");
/*
	position = m_Position;
	return PNR_OK;
*/
	return PNR_NOTIMPL;
}

/*
 * IRMASite methods called by the user of the site to get
 * information about the site, and to manipulate the site.
 */
STDMETHODIMP
CCoRTSite::SetSize(THIS_
			PNxSize			size)
{
	ATLTRACE2("Site: SetSize\n");
/*
	HRESULT hr = PNR_OK;
	if (m_SiteWatcher)
	{
		hr = m_SiteWatcher->ChangingSize(m_Size, size);
	}
	if (hr == PNR_OK)
	{
*/
		m_Size = size;
/*
		// Iterate through passive site watchers
		POSITION pos = m_PassiveSiteWatchers.GetHeadPosition();
		while (pos != NULL)
		{
			CComQIPtr<IRMAPassiveSiteWatcher, &IID_IRMAPassiveSiteWatcher> tmp_Watcher =
				m_PassiveSiteWatchers.GetNext(pos);
			tmp_Watcher->SizeChanged(&m_Size);
		}
	}
	return hr;
*/
	return PNR_OK;
}

STDMETHODIMP
CCoRTSite::GetSize(THIS_
			REF(PNxSize)		size)
{
	ATLTRACE2("Site: GetSize\n");
	size = m_Size;
	return PNR_OK;
}

STDMETHODIMP
CCoRTSite::DamageRect(THIS_
			PNxRect			rect)
{
	ATLTRACE2("Site: DamageRect\n");
	return PNR_OK;
}

STDMETHODIMP
CCoRTSite::DamageRegion(THIS_
			PNxRegion		region)
{
	ATLTRACE2("Site: DamageRegion\n");
	return PNR_OK;
}

STDMETHODIMP
CCoRTSite::ForceRedraw(THIS)
{
	ATLTRACE2("Site: ForceRedraw\n");
	PNxEvent event = {RMA_SURFACE_UPDATE, NULL, (IRMAVideoSurface*)this, NULL, 0, 0};
	m_SiteUser->HandleEvent(&event);
	return PNR_OK;
}

/*
 * IRMASite2 method usually called by the "context" 
 * when window attributes (like the window handle) have changed.
 */
STDMETHODIMP
CCoRTSite::UpdateSiteWindow(THIS_
			PNxWindow* /*IN*/ pWindow)
{
	ATLTRACE2("Site2: UpdateSiteWindow\n");
	return PNR_OK;
}

/*
 * IRMASite2 method usually called by the "context" to
 * to hide/show a site.
 */
STDMETHODIMP
CCoRTSite::ShowSite(THIS_
							 BOOL	bShow)
{
	ATLTRACE2("Site2: ShowSite\n");
	return PNR_OK;
}
							 
STDMETHODIMP_(BOOL)
CCoRTSite:: IsSiteVisible(THIS)
{
	ATLTRACE2("Site2: IsSiteVisible\n");
	return TRUE;
}

/*
 * IRMASite2 method usually called by the "context" to
 * set the site's Z-order
 */
STDMETHODIMP
CCoRTSite::SetZOrder(THIS_
			INT32 lZOrder
			)
{
	ATLTRACE2("Site2: SetZOrder\n");
	return PNR_NOTIMPL;
}

/*
 * IRMASite2 method called to get the site's Z-order
 */
STDMETHODIMP
CCoRTSite::GetZOrder(THIS_
			REF(INT32) lZOrder
			)
{
	ATLTRACE2("Site2: GetZOrder\n");
	return PNR_NOTIMPL;
}

/*
 * IRMASite2 method called to set the site at the top
 * of the Z-order
 */
STDMETHODIMP
CCoRTSite::MoveSiteToTop(THIS)
{
	ATLTRACE2("Site2: MoveSiteToTop\n");
	return PNR_NOTIMPL;
}

/*
 * IRMASite2 method called to get the site's video surface
 */
STDMETHODIMP
CCoRTSite::GetVideoSurface(THIS_ 
			REF(IRMAVideoSurface*) pSurface
			)
{
	ATLTRACE2("Site2: GetVideoSurface\n");
	pSurface = this;
	pSurface->AddRef();
	return PNR_OK;
}

/*
 * IRMASite2 method called to get the number of child sites.
 */
STDMETHODIMP_(UINT32)
CCoRTSite::GetNumberOfChildSites(THIS)
{
	ATLTRACE2("Site2: GetNumberOfChildSites\n");
	return 0;
}

/*
 * IRMASite2 method to add a watcher that does not affect the site
 */
STDMETHODIMP
CCoRTSite::AddPassiveSiteWatcher(THIS_
					IRMAPassiveSiteWatcher* pWatcher
				)
{
	ATLTRACE2("Site2: AddPassiveSiteWatcher\n");
	m_PassiveSiteWatchers.AddHead(pWatcher);
	return PNR_OK;
}

/*
 * IRMASite2 method to remove a watcher that does not affect the site
 */
STDMETHODIMP
CCoRTSite::RemovePassiveSiteWatcher(THIS_
					IRMAPassiveSiteWatcher* pWatcher
				)
{
	ATLTRACE2("Site2: RemovePassiveSiteWatcher\n");
/*
	POSITION pos = m_PassiveSiteWatchers.Find(pWatcher);
	m_PassiveSiteWatchers.RemoveAt(pos);
*/
	return PNR_OK;
}

/*
 * IRMASite2 method used to do cursor management
 */
STDMETHODIMP
CCoRTSite::SetCursor(THIS_
					PNxCursor ulCursor,
				REF(PNxCursor) ulOldCursor
				)
{
	ATLTRACE2("Site2: SetCursor\n");
	return PNR_NOTIMPL;
}

/*
 * IRMASiteWindowless methods called by owners of the site.
 */
STDMETHODIMP
CCoRTSite::EventOccurred(THIS_
			PNxEvent* /*IN*/ pEvent)
{
	ATLTRACE2("SiteWindowless: EventOccurred\n");
	return PNR_NOTIMPL;
}

/*
 * IRMASiteWindowless method. Returns some parent window that
 * owns the windowless site. Useful for right-click menus and
 * dialog box calls.
 */
STDMETHODIMP_(PNxWindow*)
CCoRTSite::GetParentWindow(THIS)
{
	ATLTRACE2("SiteWindowless: GetParentWindow\n");
	return NULL;
}

/*
 * IRMAVideoSurface methods usually called by renderers to
 * Draw on the surface
 */
STDMETHODIMP
CCoRTSite::Blt(THIS_
			UCHAR*		  /*IN*/  pImageBits,
			RMABitmapInfoHeader*	/*IN*/  pBitmapInfo,
			REF(PNxRect)		/*IN*/  rDestRect,
			REF(PNxRect)		/*IN*/  rSrcRect)
{
	ATLTRACE2("VideoSurface: Blt\n");
	HRESULT hr = BeginOptimizedBlt(pBitmapInfo);
	if (!hr)
	{
		return hr;
	}
	return OptimizedBlt(pImageBits, rDestRect, rSrcRect);
}

/************************************************************************
 *  Method:
 *	  IRMAVideoSurface::BeginOptimizedBlt
 *  Purpose:
 *	  Called by renderer to commit to a bitmap format for all future
 *	  OptimizedBlt calls.
 */
STDMETHODIMP
CCoRTSite::BeginOptimizedBlt(THIS_
			RMABitmapInfoHeader*	/*IN*/  pBitmapInfo)
{
	ATLTRACE2("VideoSurface: BeginOptimizedBlt\n");
	if (pBitmapInfo->biCompression != RMA_RGB)
	{
		return PNR_FAIL;
	}

	CopyMemory(&m_BitmapHeader, pBitmapInfo, sizeof(RMABitmapInfoHeader));
	return PNR_OK;
}

/************************************************************************
 *  Method:
 *	  IRMAVideoSurface::OptimizedBlt
 *  Purpose:
 *	  Called by renderer to draw to the video surface, in the format
 *	  previously specified by calling BeginOptimizedBlt.
 */
STDMETHODIMP
CCoRTSite::OptimizedBlt(THIS_
			UCHAR*		  /*IN*/  pImageBits,
			REF(PNxRect)		/*IN*/  rDestRect,
			REF(PNxRect)		/*IN*/  rSrcRect)
{
	ATLTRACE2("VideoSurface: OptimizedBlt\n");
	if (!m_RealThumb)
	{
		ATLTRACE2("No CoRealThumb reference!\n");
		return E_UNEXPECTED;
	}
	if (!pImageBits)
	{
		ATLTRACE2("No Image Bits!\n");
		return E_INVALIDARG;
	}

	m_BltCritSect.Lock();
	if (m_Captured)
	{
		goto ExitBlt;
	}
	// FIXME need to allocate enough for whole BITMAPINFO, not just the header
	BITMAPINFO* ret_BitmapHeader = reinterpret_cast<BITMAPINFO*>(CoTaskMemAlloc(sizeof(BITMAPINFOHEADER)));
	CopyMemory(ret_BitmapHeader, &m_BitmapHeader, sizeof(BITMAPINFOHEADER));
	UCHAR* ret_Bits = reinterpret_cast<UCHAR*>(CoTaskMemAlloc(m_BitmapHeader.biSizeImage));
	CopyMemory(ret_Bits, pImageBits, m_BitmapHeader.biSizeImage);
	ATLTRACE2("Calling Image Captured: %d/%d", ret_BitmapHeader, ret_Bits);
	m_RealThumb->ImageCaptured(ret_BitmapHeader, ret_Bits);
	m_Captured = TRUE;
ExitBlt:
	m_BltCritSect.Unlock();
	return PNR_OK;
}

/************************************************************************
 *  Method:
 *	  IRMAVideoSurface::EndOptimizedBlt
 *  Purpose:
 *	  Called by renderer allow the video surface to cleanup after all
 *	  OptimizedBlt calls have been made.
 */
STDMETHODIMP
CCoRTSite::EndOptimizedBlt(THIS)
{
	ATLTRACE2("VideoSurface: EndOptimizedBlt\n");
	ZeroMemory(&m_BitmapHeader, sizeof(RMABitmapInfoHeader));
	return PNR_OK;
}

/************************************************************************
 *  Method:
 *	  IRMAVideoSurface::GetOptimizedFormat
 *  Purpose:
 *	  Called by the client to find out what compression type the
 *	  renderer committed to when it called BeginOptimizedBlt.
 */
STDMETHODIMP
CCoRTSite::GetOptimizedFormat(THIS_
			REF(RMA_COMPRESSION_TYPE) /*OUT*/ ulType)
{
	ATLTRACE2("VideoSurface: GetOptimizedFormat\n");
	ulType = m_BitmapHeader.biCompression;
	return PNR_OK;
}

/************************************************************************
 *  Method:
 *	  IRMAVideoSurface::GetPreferredFormat
 *  Purpose:
 *	  Called by renderer to find out what compression type the video
 *	  surface would prefer to be given in BeginOptimizedBlt.
 */
STDMETHODIMP
CCoRTSite::GetPreferredFormat(THIS_
			REF(RMA_COMPRESSION_TYPE) /*OUT*/ ulType)
{
	ATLTRACE2("VideoSurface: GetPreferredFormat\n");
	ulType = RMA_RGB;
	return PNR_OK;
}