/*
 * RealThumb v0.21
 * CoRTSite.h - RealPlayer Site header
 * Copyright (c) 2003 Michael Barrientos
 * All rights reserved.
 * http://mb.n3.net/projects/
 */

// CoRTSite.h : Declaration of the CCoRTSite

#pragma once
#include "resource.h"	   // main symbols

#include "RealThumb.h"
#include "rmawin.h"
#include "rmasite2.h"
#include "rmavsurf.h"

// CCoRTSite

class ATL_NO_VTABLE CCoRTSite : 
	public CComObjectRootEx<CComObjectThreadModel>,
	public CComCoClass<CCoRTSite, &CLSID_CoRTSite>,
	public ICoRTSite,
	public IRMASite,
	public IRMASite2,
	public IRMASiteWindowless,
	public IRMAVideoSurface
{
protected:
	CComQIPtr<IRMASiteUser, &IID_IRMASiteUser> m_SiteUser;
	CComQIPtr<IRMASiteWatcher, &IID_IRMASiteWatcher> m_SiteWatcher;
	CComQIPtr<IRMAValues, &IID_IRMAValues> m_Values;
	CComPtr<ICoRealThumb> m_RealThumb;
	volatile BOOL m_Captured;
	CComAutoCriticalSection m_BltCritSect;
	CInterfaceList<IRMAPassiveSiteWatcher, &IID_IRMAPassiveSiteWatcher> m_PassiveSiteWatchers;
	PNxSize m_Size;
/*
	PNxPoint m_Position;
*/
	RMABitmapInfoHeader m_BitmapHeader;
public:
	CCoRTSite(): m_Captured(FALSE)
	{
		m_Size.cx = 0;
		m_Size.cy = 0;
/*
		m_Position.x = 0;
		m_Position.y = 0;
*/
	    memset(&m_BitmapHeader, 0, sizeof(RMABitmapInfoHeader));
	}

DECLARE_REGISTRY_RESOURCEID(IDR_CORTSITE)


BEGIN_COM_MAP(CCoRTSite)
	COM_INTERFACE_ENTRY(ICoRTSite)
	COM_INTERFACE_ENTRY_IID(IID_IRMASite, IRMASite)
	COM_INTERFACE_ENTRY_IID(IID_IRMASite2, IRMASite2)
	COM_INTERFACE_ENTRY_IID(IID_IRMASiteWindowless, IRMASiteWindowless)
	COM_INTERFACE_ENTRY_IID(IID_IRMAVideoSurface, IRMAVideoSurface)
	COM_INTERFACE_ENTRY_AGGREGATE_BLIND(m_Values.p)
END_COM_MAP()


	DECLARE_PROTECT_FINAL_CONSTRUCT()

	HRESULT FinalConstruct()
	{
		return S_OK;
	}
	
	void FinalRelease() 
	{
		m_SiteUser = reinterpret_cast<IUnknown*>(NULL);
		m_SiteWatcher = reinterpret_cast<IUnknown*>(NULL);
		m_Values = reinterpret_cast<IUnknown*>(NULL);
		m_RealThumb = NULL;
		m_PassiveSiteWatchers.RemoveAll();
	}

public:
	/* ICoRTSite methods */
	STDMETHOD(Init) (THIS_
				IUnknown *pContext,
				ICoRealThumb *pRealThumb);
	/*
	 * IRMASite methods usually called by the "context" to 
	 * associate users with the site, and to create child sites
	 * as appropriate.
	 */
	STDMETHOD(AttachUser)	(THIS_
				IRMASiteUser*	/*IN*/	pUser);

	STDMETHOD(DetachUser)	(THIS);


	STDMETHOD(GetUser)		(THIS_
				REF(IRMASiteUser*) /*OUT*/ pUser);

	STDMETHOD(CreateChild)	(THIS_
				REF(IRMASite*)	/*OUT*/ pChildSite);

	STDMETHOD(DestroyChild)	(THIS_
				IRMASite*	/*IN*/	pChildSite);

	/*
	 * IRMASite methods called by the the "context" in which the site
	 * is displayed in order to manage its position. Site users should
	 * not generally call these methods.
	 */
	STDMETHOD(AttachWatcher)	(THIS_
				IRMASiteWatcher* /*IN*/	pWatcher);

	STDMETHOD(DetachWatcher)	(THIS);

	STDMETHOD(SetPosition)	(THIS_
				PNxPoint		position);

	STDMETHOD(GetPosition)	(THIS_
				REF(PNxPoint)		position);

	/*
	 * IRMASite methods called by the user of the site to get
	 * information about the site, and to manipulate the site.
	 */
	STDMETHOD(SetSize)		(THIS_
				PNxSize			size);

	STDMETHOD(GetSize)		(THIS_
				REF(PNxSize)		size);

	STDMETHOD(DamageRect)	(THIS_
				PNxRect			rect);

	STDMETHOD(DamageRegion)	(THIS_
				PNxRegion		region);

	STDMETHOD(ForceRedraw)	(THIS);

	/*
	 * IRMASite2 method usually called by the "context" 
	 * when window attributes (like the window handle) have changed.
	 */
	STDMETHOD(UpdateSiteWindow) (THIS_
				PNxWindow* /*IN*/ pWindow);

	/*
	 * IRMASite2 method usually called by the "context" to
	 * to hide/show a site.
	 */
	STDMETHOD(ShowSite)		 (THIS_
								 BOOL	bShow);
								 
	STDMETHOD_(BOOL, IsSiteVisible)		 (THIS);

	/*
	 * IRMASite2 method usually called by the "context" to
	 * set the site's Z-order
	 */
	STDMETHOD(SetZOrder)	(THIS_
				INT32 lZOrder
				);

	/*
	 * IRMASite2 method called to get the site's Z-order
	 */
	STDMETHOD(GetZOrder)	(THIS_
				REF(INT32) lZOrder
				);

	/*
	 * IRMASite2 method called to set the site at the top
	 * of the Z-order
	 */
	STDMETHOD(MoveSiteToTop)	(THIS);

	/*
	 * IRMASite2 method called to get the site's video surface
	 */
	STDMETHOD(GetVideoSurface)	(THIS_ 
				REF(IRMAVideoSurface*) pSurface
				);

	/*
	 * IRMASite2 method called to get the number of child sites.
	 */
	STDMETHOD_(UINT32,GetNumberOfChildSites) (THIS);

	/*
	 * IRMASite2 method to add a watcher that does not affect the site
	 */
	STDMETHOD(AddPassiveSiteWatcher)	(THIS_
						IRMAPassiveSiteWatcher* pWatcher
					);

	/*
	 * IRMASite2 method to remove a watcher that does not affect the site
	 */
	STDMETHOD(RemovePassiveSiteWatcher) (THIS_
						IRMAPassiveSiteWatcher* pWatcher
					);

	/*
	 * IRMASite2 method used to do cursor management
	 */
	STDMETHOD(SetCursor) 		(THIS_
						PNxCursor ulCursor,
					REF(PNxCursor) ulOldCursor
					);

	/*
	 * IRMASiteWindowless methods called by owners of the site.
	 */
	STDMETHOD(EventOccurred)	(THIS_
				PNxEvent* /*IN*/ pEvent);

	/*
	 * IRMASiteWindowless method. Returns some parent window that
	 * owns the windowless site. Useful for right-click menus and
	 * dialog box calls.
	 */
	STDMETHOD_(PNxWindow*,GetParentWindow)(THIS);

	/*
	 * IRMAVideoSurface methods usually called by renderers to
	 * Draw on the surface
	 */
	STDMETHOD(Blt)	  (THIS_
				UCHAR*		  /*IN*/  pImageBits,
				RMABitmapInfoHeader*	/*IN*/  pBitmapInfo,
				REF(PNxRect)		/*IN*/  rDestRect,
				REF(PNxRect)		/*IN*/  rSrcRect);

	/************************************************************************
	 *  Method:
	 *	  IRMAVideoSurface::BeginOptimizedBlt
	 *  Purpose:
	 *	  Called by renderer to commit to a bitmap format for all future
	 *	  OptimizedBlt calls.
	 */
	STDMETHOD(BeginOptimizedBlt)(THIS_
				RMABitmapInfoHeader*	/*IN*/  pBitmapInfo);

	/************************************************************************
	 *  Method:
	 *	  IRMAVideoSurface::OptimizedBlt
	 *  Purpose:
	 *	  Called by renderer to draw to the video surface, in the format
	 *	  previously specified by calling BeginOptimizedBlt.
	 */
	STDMETHOD(OptimizedBlt) (THIS_
				UCHAR*		  /*IN*/  pImageBits,
				REF(PNxRect)		/*IN*/  rDestRect,
				REF(PNxRect)		/*IN*/  rSrcRect);

	/************************************************************************
	 *  Method:
	 *	  IRMAVideoSurface::EndOptimizedBlt
	 *  Purpose:
	 *	  Called by renderer allow the video surface to cleanup after all
	 *	  OptimizedBlt calls have been made.
	 */
	STDMETHOD(EndOptimizedBlt)  (THIS);

	/************************************************************************
	 *  Method:
	 *	  IRMAVideoSurface::GetOptimizedFormat
	 *  Purpose:
	 *	  Called by the client to find out what compression type the
	 *	  renderer committed to when it called BeginOptimizedBlt.
	 */
	STDMETHOD(GetOptimizedFormat)(THIS_
				REF(RMA_COMPRESSION_TYPE) /*OUT*/ ulType);

	/************************************************************************
	 *  Method:
	 *	  IRMAVideoSurface::GetPreferredFormat
	 *  Purpose:
	 *	  Called by renderer to find out what compression type the video
	 *	  surface would prefer to be given in BeginOptimizedBlt.
	 */
	STDMETHOD(GetPreferredFormat)(THIS_
				REF(RMA_COMPRESSION_TYPE) /*OUT*/ ulType);
};

OBJECT_ENTRY_AUTO(__uuidof(CoRTSite), CCoRTSite)
