/*
 * RealThumb v0.21
 * CoRealThumb.cpp - Thumbnail extractor implementation
 * Copyright (c) 2003 Michael Barrientos
 * All rights reserved.
 * http://mb.n3.net/projects/
 */

// CoRealThumb.cpp : Implementation of CCoRealThumb

#include "stdafx.h"
#include "CoRealThumb.h"
#include "CoRTNullAudioDevice.h"
#include "CoRTContext.h"

// CCoRealThumb

// TODO: Add caching
HRESULT CCoRealThumb::initializeRT()
{
	USES_CONVERSION;
	HRESULT retVal = S_OK;
	m_CritSectRT.Lock();
	if (!rtInitialized)
	{
		// prepare/load the RMACore module
		_TCHAR szDllName[_MAX_PATH];

		// get location of rmacore from windows registry 
		ULONG bufSize;

		CRegKey tmp_RegKey;
    
		szDllName[0] = '\0'; 
		bufSize = sizeof(szDllName) - 1;

		if (tmp_RegKey.Open(HKEY_CLASSES_ROOT,
			"Software\\RealNetworks\\Preferences\\DT_Common", KEY_READ) == ERROR_SUCCESS)
		{ 
			// We don't care if this fails, since if this fails, so will the DLL loading
			// No need to save return value
			tmp_RegKey.QueryStringValue("", szDllName, &bufSize);
			tmp_RegKey.Close();
		}

		StrCat(szDllName, "pnen3260.dll");
    

		if (!(hDll = CoLoadLibrary(T2OLE(szDllName), FALSE)))
		{
			retVal = E_FAIL;
			goto initializeExit;
		}

		FPRMCREATEENGINE tmp_DllCreateEngine = NULL;
		if (!(tmp_DllCreateEngine = (FPRMCREATEENGINE) GetProcAddress(hDll, "CreateEngine")))
		{
			retVal = E_FAIL;
			goto initializeExit;
		}

		if (PNR_OK != tmp_DllCreateEngine(&m_Engine.p))
		{
			retVal = E_FAIL;
			goto initializeExit;
		}
		m_Engine.p->AddRef();

		// replace the audio device before we initialize the player
		m_AudioDevice = new CCOMOBJECT<CCoRTNullAudioDevice>;
		if (m_AudioDevice)
		{
			CComQIPtr<IRMAAudioDeviceManager,
				&IID_IRMAAudioDeviceManager> tmp_AudioDeviceManager = m_Engine;
			if (tmp_AudioDeviceManager)
			    if (tmp_AudioDeviceManager->Replace(m_AudioDevice) != PNR_OK)
					m_AudioDevice = reinterpret_cast<IUnknown*>(NULL);
		}

		rtInitialized = TRUE;
	}
initializeExit:
	m_CritSectRT.Unlock();
	return retVal;
}

STDMETHODIMP CCoRealThumb::GetClassID(LPCLSID lpClassID)
{
	lpClassID = &((CLSID)GetObjectCLSID());
	return S_OK;
}

STDMETHODIMP CCoRealThumb::IsDirty()
{
    return S_FALSE;
}

STDMETHODIMP CCoRealThumb::Load(LPCOLESTR lpszFileName, DWORD grfMode)
{
	ATLTRACE2("Load: Running!\n");
	// LPCOLESTR always Unicode, I think
	if (!lpszFileName)
	{
		ATLTRACE2("Load: Bad Pointer!\n");
		return E_POINTER;
	}

	m_CurrentState = IRTIR_TASK_NOT_RUNNING;

	m_CritSectIPersist.Lock();
	if (m_FileName)
	{
		LocalFree(m_FileName);
	}
	m_FileName = StrDupW(lpszFileName);
	if (!m_FileName)
	{
		ATLTRACE2("Load: Couldn't duplicate filename!\n");
		return E_OUTOFMEMORY;
	}
	m_CritSectIPersist.Unlock();
//	Ignoring access mode
//	m_GrfMode = grfMode;
	ATLTRACE2("Load: Exiting function.\n");
	return S_OK;
}

STDMETHODIMP CCoRealThumb::Save(LPCOLESTR lpszFileName, BOOL fRemember)
{
    return E_FAIL;
}

STDMETHODIMP CCoRealThumb::SaveCompleted(LPCOLESTR lpszFileName)
{
    return E_FAIL;
}

STDMETHODIMP CCoRealThumb::GetCurFile(LPOLESTR* ppszFileName)
{
	HRESULT retVal = S_OK;
	// No need to implement, since no saving implemented (?)
	// But I did anyways

	if (!ppszFileName)
	{
		return E_POINTER;
	}

	m_CritSectIPersist.Lock();
	size_t size = sizeof(OLECHAR) * (m_FileName == NULL ? 1 : wcslen(m_FileName) + 1);
	LPOLESTR returnName = (LPOLESTR)CoTaskMemAlloc(size);
	if (returnName == NULL)
	{
		retVal = E_OUTOFMEMORY;
		goto ExitCurFile;
	}

	// Default save string is just empty string for now.
	if (m_FileName == NULL)
	{
		returnName[0] = '\0';
		retVal = S_FALSE;
	} else
	{
		StrCpyW(returnName, m_FileName);
	}
	*ppszFileName = returnName;
ExitCurFile:
	m_CritSectIPersist.Unlock();
	return retVal;
}

STDMETHODIMP CCoRealThumb::GetLocation(LPWSTR pszPathBuffer, DWORD cchMax, DWORD *pdwPriority,
	const SIZE *prgSize, DWORD dwRecClrDepth, DWORD *pdwFlags)
{
	ATLTRACE2("GetLocation: Running!\n");
	if (!m_FileName)
	{
		ATLTRACE2("GetLocation: No File Name!\n");
		return E_POINTER;
	}
	if (!pdwFlags)
	{
		ATLTRACE2("GetLocation: Bad flags!\n");
	}

	m_CritSectIPersist.Lock();

	StrCpyNW(pszPathBuffer, m_FileName, cchMax);
	m_CritSectIPersist.Unlock();

	m_CritSectIExtract.Lock();
	CopyMemory(&m_Size, prgSize, sizeof(SIZE));
	m_ClrDepth = dwRecClrDepth;
	m_Flags = *pdwFlags;
	m_CritSectIExtract.Unlock();

	*pdwFlags |= IEIFLAG_CACHE;
	ATLTRACE2("GetLocation: Exiting function.\n");
	if (m_Flags & IEIFLAG_ASYNC)
	{
		if (pdwPriority)
		{
			*pdwPriority = 0;
		}
		return E_PENDING;
	}
	return NOERROR;
}

STDMETHODIMP CCoRealThumb::Extract(HBITMAP *phBmpImage)
{
	USES_CONVERSION;
	CComQIPtr<IRMAPlayer, &IID_IRMAPlayer> tmp_Player(reinterpret_cast<IUnknown*>(NULL));
	CComQIPtr<ICoRTContext> tmp_Context(reinterpret_cast<IUnknown*>(NULL));
	LPTSTR tmp_FileName(NULL);
	LPSTR pszURL(NULL);
	*phBmpImage = NULL;

	ATLTRACE2("Extract: Running!\n");
	HRESULT retVal = NOERROR;
	// TODO: add draw code for native data here
	m_CritSectIPersist.Lock();
	if (!m_FileName)
	{
		ATLTRACE2("Extract: No File Name!\n");
		m_CritSectIPersist.Unlock();
		return E_UNEXPECTED;
	}
	m_CurrentState = IRTIR_TASK_RUNNING;

	if (initializeRT() != S_OK)
	{
		ATLTRACE2("Extract: Error loading DLL!\n");
		m_CritSectIPersist.Unlock();
		retVal = E_FAIL;
		goto ExitExtract1;
	}

	tmp_FileName = OLE2T(m_FileName);
	m_CritSectIPersist.Unlock();
	_TCHAR tmp_Url[_MAX_PATH + 4] = "file:";
	StrCatN(tmp_Url, tmp_FileName, _MAX_PATH + 4);
	_TCHAR tmp_UrlEscape[_MAX_PATH * 3 + 4];
	DWORD lenUrl = _MAX_PATH * 3 + 4;
	retVal = UrlEscape(tmp_Url, tmp_UrlEscape, &lenUrl, URL_ESCAPE_PERCENT);
	if (retVal != S_OK)
	{
		goto ExitExtract1;
	}
	pszURL = T2A(tmp_UrlEscape);

	// create player
	retVal = m_Engine->CreatePlayer(tmp_Player.p);
	if (PNR_OK != retVal)
	{
		goto ExitExtract1;
	}
	tmp_Player.p->AddRef();

	tmp_Context = new CCOMOBJECT<CCoRTContext>;
	retVal = tmp_Context->Init(tmp_Player, this);
	if (retVal != S_OK)
	{
		goto ExitExtract1;
	}
	tmp_Player->SetClientContext(tmp_Context);

	if (PNR_OK == tmp_Player->OpenURL(pszURL))
	{
		MSG msg;
		m_Captured = FALSE;
		tmp_Player->Begin();
		while (GetMessage(&msg, 0, 0, 0) > 0 && !tmp_Player->IsDone() && !m_Captured)
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		tmp_Player->Stop();
		// while (!tmp_Player->IsDone());
	}
	m_Engine->ClosePlayer(tmp_Player);
	// FIXME!

	if (m_Bits == NULL)
	{
		retVal = E_FAIL;
		goto ExitExtract1;
	}

	m_CritSectIExtract.Lock();

	// Dump on to clipboard for debug purposes
	// OpenClipboard(NULL);
	// EmptyClipboard();
	// SetClipboardData(CF_DIB, diBitmap);
	// CloseClipboard();

	// Convert DIB to DDB by drawing the DIB into another device context, then
	// extracting the DDB from the context.

	// First, decipher the DIB and get pointers to the appropriate fields.

	// Determine the size of the returned bitmap. We want to use as much of the space
	// as possible without distorting the aspect ratio.
	// FIXME check cx & cy for 0 (avoid division by 0)
	SIZE tmp_Size;
	if ((float)m_Size.cx/m_Size.cy >
		(float)m_Info->bmiHeader.biWidth/m_Info->bmiHeader.biHeight)
	{
		// Destination
		tmp_Size.cx = m_Info->bmiHeader.biWidth * m_Size.cy / m_Info->bmiHeader.biHeight;
		tmp_Size.cy = m_Size.cy;
	} else
	{
		tmp_Size.cx = m_Size.cx;
		tmp_Size.cy = m_Info->bmiHeader.biHeight * m_Size.cx / m_Info->bmiHeader.biWidth;
	}

	// Draw DIB into bitmap
	// First create a device context compatible without our window, select a bitmap into
	// it, then render the DIB (aka Stretch the DIB bits) into the bitmap at the
	// appropriate size. Return that bitmap we rendered into, and we're done!
	
	HDC tmpDC = CreateCompatibleDC(GetWindowDC(NULL));
	if (!tmpDC)
	{
		ATLTRACE2("Extract: Failed on CreateCompatibleDC\n");
		retVal = E_OUTOFMEMORY;
		goto ExitExtract2;
	}
	ATLTRACE2("DC Num Colors: %d, Num Bits: %d\n", GetDeviceCaps(tmpDC, NUMCOLORS), GetDeviceCaps(tmpDC, BITSPIXEL));
	
// FIXME!
	*phBmpImage = CreateCompatibleBitmap(GetWindowDC(NULL), tmp_Size.cx, tmp_Size.cy);
//	*phBmpImage = CreateBitmap(tmp_Size.cx, tmp_Size.cy, 1, m_ClrDepth, NULL);
	if (!*phBmpImage)
	{
		ATLTRACE2("Extract: Failed on CreateBitmap, or SelectObject\n");
		retVal = E_OUTOFMEMORY;
		goto ExitExtract2;
	}
	HGDIOBJ tmpObj = SelectObject(tmpDC, *phBmpImage);
	if (!tmpObj)
	{
		ATLTRACE2("Extract: Failed on SelectObject\n");
		*phBmpImage = NULL;
		retVal = E_FAIL;
		goto ExitExtract2;
	}
	// FIXME Document bad quality in Win95/98/ME because of lack of halftone support
	if (SetStretchBltMode(tmpDC, HALFTONE) == 0)
	{
		ATLTRACE2("Could not switch to halftone stretching. Be prepared for shitty output!");
	}
	StretchDIBits(tmpDC, 0, 0, tmp_Size.cx, tmp_Size.cy,
		0, 0, m_Info->bmiHeader.biWidth, m_Info->bmiHeader.biHeight,
		m_Bits, m_Info, DIB_PAL_COLORS, SRCCOPY);

	// Dump bitmap into clipboard for debug purposes
	// OpenClipboard(NULL);
	// EmptyClipboard();
	// SetClipboardData(CF_BITMAP, *phBmpImage);
	// CloseClipboard();

	// Cleanup
	SelectObject(tmpDC, tmpObj);
	DeleteDC(tmpDC);
	CoTaskMemFree(m_Bits);
	CoTaskMemFree(m_Info);
	m_Bits = NULL;
	m_Info = NULL;

ExitExtract2:
	m_CritSectIExtract.Unlock();
ExitExtract1:
	m_CurrentState = IRTIR_TASK_FINISHED;
	ATLTRACE2("Extract: Exiting function.\n");
	return retVal;
}

STDMETHODIMP_(ULONG) CCoRealThumb::IsRunning() {
	ATLTRACE2("IsRunning: Running!\n");
	return m_CurrentState;
}

STDMETHODIMP CCoRealThumb::Run()
{
	return E_NOTIMPL;
}

STDMETHODIMP CCoRealThumb::Suspend()
{
	return E_NOTIMPL;
}

STDMETHODIMP CCoRealThumb::Kill(BOOL fUnused)
{
	return E_NOTIMPL;
}

STDMETHODIMP CCoRealThumb::Resume()
{
	return E_NOTIMPL;
}

STDMETHODIMP CCoRealThumb::ImageCaptured(LPBITMAPINFO pInfo, PUCHAR pBits)
{
	m_Info = pInfo;
	m_Bits = pBits;
	m_Captured = TRUE;
	return S_OK;
}
