/*
 * RealThumb v0.21
 * CoRealThumb.h - Thumbnail extractor header
 * Copyright (c) 2003 Michael Barrientos
 * All rights reserved.
 * http://mb.n3.net/projects/
 */

// CoRealThumb.h : Declaration of the CCoRealThumb

#pragma once
#include "resource.h"       // main symbols

#include "RealThumb.h"


// CCoRealThumb

class ATL_NO_VTABLE CCoRealThumb : 
	public CComObjectRootEx<CComObjectThreadModel>,
	public CComCoClass<CCoRealThumb, &CLSID_CoRealThumb>,
	public ICoRealThumb,
	public IPersistFile,
	public IExtractImage,
	public IRunnableTask
{
public:
	CCoRealThumb() :
		m_FileName(NULL),
		m_CurrentState(IRTIR_TASK_NOT_RUNNING),
		m_Info(NULL),
		m_Bits(NULL)
	{
		ATLTRACE2("Created RealThumb\n");
		CComObjectThreadModel::Increment(&m_RtCount);
	}

DECLARE_REGISTRY_RESOURCEID(IDR_COREALTHUMB)

// typedef ATL::CComCreator2< ATL::CComCreator< CComDebugObject< CCoRealThumb > >, ATL::CComCreator< ATL::CComAggObject< CCoRealThumb > > > _CreatorClass;

BEGIN_COM_MAP(CCoRealThumb)
	COM_INTERFACE_ENTRY(ICoRealThumb)
	COM_INTERFACE_ENTRY(IPersistFile)
	COM_INTERFACE_ENTRY(IExtractImage)
	COM_INTERFACE_ENTRY_IID(IID_IRunnableTask, IRunnableTask)
END_COM_MAP()


	DECLARE_PROTECT_FINAL_CONSTRUCT()

	HRESULT FinalConstruct()
	{
		return S_OK;
	}
	
	void FinalRelease() 
	{
		ATLTRACE2("Destroyed\n");

		m_CritSectIPersist.Lock();
		if (m_FileName)
		{
			LocalFree(m_FileName);
		}
		m_CritSectIPersist.Unlock();

		ATLTRACE2("RT Kill\n");

		// Yield to allow other threads to process before we uninitialize RT.
		m_CritSectRT.Lock();
		if ((CComObjectThreadModel::Decrement(&m_RtCount) == 0) && rtInitialized)
		{
			if (m_AudioDevice)
			{
				CComQIPtr<IRMAAudioDeviceManager, &IID_IRMAAudioDeviceManager> tmp_AudioDeviceManager = m_Engine;
				if (tmp_AudioDeviceManager)
				{
				    tmp_AudioDeviceManager->Remove(m_AudioDevice);
				}
				m_AudioDevice = reinterpret_cast<IUnknown*>(NULL);
		    }

			FPRMCLOSEENGINE tmp_DllCloseEngine =
				(FPRMCLOSEENGINE) GetProcAddress(hDll, "CloseEngine");
			tmp_DllCloseEngine(m_Engine);
			m_Engine = reinterpret_cast<IUnknown*>(NULL);

			CoFreeLibrary(hDll);
			hDll = NULL;
			rtInitialized = FALSE;
		}
		m_CritSectRT.Unlock();
	}

protected:
	volatile LONG m_CurrentState;
	static LONG m_RtCount;
	static BOOL rtInitialized;
	static HINSTANCE hDll;
	static CComQIPtr<IRMAClientEngine, &IID_IRMAClientEngine> m_Engine;
	static CComQIPtr<IRMAAudioDevice, &IID_IRMAAudioDevice> m_AudioDevice;

	volatile BOOL m_Captured;
	CComAutoCriticalSection m_CritSectRT;

	// Protects variables set by IExtractImage::GetLocation
	CComAutoCriticalSection m_CritSectIExtract;

	// Protects variables set by IPersistFile::Load
	CComAutoCriticalSection m_CritSectIPersist;

	//ICoRealThumb variables
	LPBITMAPINFO m_Info;
	PUCHAR m_Bits;

	//IPersistFile variables
	LPOLESTR m_FileName;
	// DWORD m_GrfMode;

	//IExtractImage variables
	SIZE m_Size;
	DWORD m_ClrDepth;
	DWORD m_Flags;

	HRESULT initializeRT();
public:
	//ICoRealThumb methods
	STDMETHOD(ImageCaptured)(LPBITMAPINFO pInfo, PUCHAR pBits);

	//IRunnableTask methods
	STDMETHOD_(ULONG,IsRunning)();
	STDMETHOD(Kill)(BOOL fUnused);
	STDMETHOD(Run)();
	STDMETHOD(Resume)();
	STDMETHOD(Suspend)();

	//IPersistFile methods
	STDMETHOD(GetClassID)(LPCLSID lpClassID);
	STDMETHOD(IsDirty)();
	STDMETHOD(Load)(LPCOLESTR lpszFileName, DWORD grfMode);
	STDMETHOD(Save)(LPCOLESTR lpszFileName, BOOL fRemember);
	STDMETHOD(SaveCompleted)(LPCOLESTR lpszFileName);
	STDMETHOD(GetCurFile)(LPOLESTR* ppszFileName);

	//IExtractImage methods
	STDMETHOD(GetLocation)(LPWSTR pszPathBuffer,DWORD cchMax,DWORD *pdwPriority,
		const SIZE *prgSize,DWORD dwRecClrDepth,DWORD *pdwFlags);
	STDMETHOD(Extract)(HBITMAP *phBmpImage);
};
BOOL CCoRealThumb::rtInitialized = FALSE;
LONG CCoRealThumb::m_RtCount = 0;
HINSTANCE CCoRealThumb::hDll = NULL;
CComQIPtr<IRMAClientEngine, &IID_IRMAClientEngine> CCoRealThumb::m_Engine;
CComQIPtr<IRMAAudioDevice, &IID_IRMAAudioDevice> CCoRealThumb::m_AudioDevice;

OBJECT_ENTRY_AUTO(__uuidof(CoRealThumb), CCoRealThumb)
